<?php
/**
* @copyright Copyright (C) 2006 - 2010 MightyExtensions (http://www.mightyextensions.com). All rights reserved.
* @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
**/
defined('_JEXEC') or die( 'Restricted access' );

jimport('joomla.application.component.model');

class ReportModelReport extends JModel
{
	var $_data = null;
	var $_total = null;
	var $_pagination = null;
	var $_id = null;

	function __construct()
	{
		parent::__construct();

		global $mainframe, $option;

		// Get the pagination request variables
		$limit		= $mainframe->getUserStateFromRequest( 'global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int' );
		$limitstart	= $mainframe->getUserStateFromRequest( $option.'.limitstart', 'limitstart', 0, 'int' );

		// In case limit has been changed, adjust limitstart accordingly
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);

		$array = JRequest::getVar('cid', array(0), '', 'array');
		$this->setId((int)$array[0]);
	}
	function setId($id)
	{
		// Set weblink id and wipe data
		$this->_id		= $id;
		$this->_data	= null;
	}
	function isCheckedOut( $report, $uid = 0 )
	{
		if ($uid) {
			return ($report->checked_out && $report->checked_out != $uid);
		} else {
			return $report->checked_out;
		}
	}
	function checkin($id)
	{
		$record = & $this->getTable('report','Report');
		if(!$record->checkin($id)) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}
		return false;
	}
	function checkout($uid = null)
	{
		if ($this->_id)
		{
			// Make sure we have a user id to checkout the article with
			if (is_null($uid)) {
				$user	=& JFactory::getUser();
				$uid	= $user->get('id');
			}
			// Lets get to it and checkout the thing...
			$record = & $this->getTable('Report','Report');
			if(!$record->checkout($uid, $this->_id)) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}

			return true;
		}
		return false;
	}

	function getReport()
	{
		$cid = JRequest::getVar('cid', array(0), 'request', 'array');
		$table = JTable::getInstance('Report', 'Report');
		$table->load($cid[0]);
		return $table;
	}

	function changeAccess($uid, $access) {
		$report = JTable::getInstance('Report', 'Report');
		$report->load ( $uid );
		$report->access = $access;

		if (! $report->check ()) {
			return $report->getError ();
		}
		if (! $report->store ()) {
			return $report->getError ();
		}
	}

	function getGroupsOptions()
	{
		global $mainframe, $option;

		$this->_db->setQuery( "   SELECT `folder` AS value, REPLACE( `folder`, '_', ' ') AS text"
		."\n     FROM #__plugins"
		."\n    WHERE published = 1 AND folder LIKE 'reports%'"
		."\n GROUP BY folder"
		."\n ORDER BY folder ASC" );
			
		$this->_db->setQuery( "   SELECT `element` AS value, name AS text "
		."\n     FROM #__plugins"
		."\n    WHERE published = 1 AND folder = 'analytics'"
		."\n ORDER BY name ASC" );
			
		$options = $this->_db->loadObjectList();

		array_unshift( $options, JHTML::_( 'select.option', '', '- Select Group -' ) );
		return $options;
	}

	function getData()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_data))
		{
			$query = $this->_buildQuery();
			$this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
		}

		return $this->_data;
	}

	function getTotal()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
		}

		return $this->_total;
	}

	function getPagination()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
		}

		return $this->_pagination;
	}

	function _buildQuery()
	{
		// Get the WHERE and ORDER BY clauses for the query
		$where		= $this->_buildContentWhere();
		$orderby	= $this->_buildContentOrderBy();

		$query = ' SELECT a.*, cc.title, g.name as groupname
		FROM #__me_analytics a
		LEFT JOIN #__categories AS cc ON cc.id = a.catid
		LEFT JOIN #__groups AS g ON g.id = a.access'
		. $where
		. $orderby
		;
		return $query;
	}

	function _buildContentOrderBy()
	{
		global $mainframe, $option;

		$filter_order	  = $mainframe->getUserStateFromRequest( $option.'filter_order',	 'filter_order',	 'a.ordering', 'cmd' );
		$filter_order_Dir = $mainframe->getUserStateFromRequest( $option.'filter_order_Dir', 'filter_order_Dir', 'DESC',	 'word' );

		if ($filter_order == 'a.ordering'){
			$orderby 	= ' ORDER BY cc.ordering ASC, a.ordering '.$filter_order_Dir;
		} else {
			$orderby 	= ' ORDER BY '.$filter_order.' '.$filter_order_Dir.' , a.ordering ';
		}

		return $orderby;
	}

	function _buildContentWhere()
	{
		global $mainframe, $option;

		$db	=& JFactory::getDBO();
		$filter_catid		= $mainframe->getUserStateFromRequest( $option.'filter_catid',		'filter_catid',		0,				'int' );
		$filter_state		= $mainframe->getUserStateFromRequest( $option.'filter_state',	'filter_state',	'',	'word' );
		$search       = $mainframe->getUserStateFromRequest( $option.'search',       'search',       '',         'string' );
		$search       = JString::strtolower( $search );

		$where = array();
		if($filter_state != '')
		$where[] = "a.published = '".($filter_state == 'P' ? 1 : 0)."'";

		if ( $search ) {
			$where[] = 'LOWER(a.name) LIKE '.$db->Quote( '%'.$db->getEscaped( $search, true ).'%', false );
		}
		if($filter_catid)
		$where[] = "a.catid = '$filter_catid'";

		$where = ( count( $where ) ? ' WHERE '. implode( ' AND ', $where ) : '' );

		return $where;
	}

	function publish($cid = array(), $publish = 1)
	{
		$user 	=& JFactory::getUser();

		if (count( $cid ))
		{
			JArrayHelper::toInteger($cid);
			$cids = implode( ',', $cid );

			$query = 'UPDATE #__me_analytics'
			. ' SET published = '.(int) $publish
			. ' WHERE id IN ( '.$cids.' )'
			. ' AND ( checked_out = 0 OR ( checked_out = '.(int) $user->get('id').' ) )'
			;
			$this->_db->setQuery( $query );

			if (!$this->_db->query()) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}
		}

		return true;
	}

	function saveorder($cid = array(), $order)
	{
		$row = JTable::getInstance("report", 'Report');
		$groupings = array();

		// update ordering values
		for( $i=0; $i < count($cid); $i++ )
		{
			$row->load( (int) $cid[$i] );
			// track categories
			$groupings[] = $row->catid;

			if ($row->ordering != $order[$i])
			{
				$row->ordering = $order[$i];
				if (!$row->store()) {
					$this->setError($this->_db->getErrorMsg());
					return false;
				}
			}
		}

		// execute updateOrder for each parent group
		$groupings = array_unique( $groupings );
		foreach ($groupings as $group){
			$row->reorder('catid = '.(int) $group);
		}

		return true;
	}

}
