<?php
/**
* @copyright Copyright (C) 2006 - 2010 MightyExtensions (http://www.mightyextensions.com). All rights reserved.
* @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
**/

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.utilities.simplexml');

class AmChart extends JObject {

	var $swfObjectPath = "swfobject.js";
	var $libraryPath = "amchart/";
	var $xmlSettingsFile = '';

	var $id = null;
	var $config = array();
	var $labels = array();

	/**
	 * Constructor can only be called from derived class because AmChart
	 * is abstract.
	 *
	 * @param	string				$_id
	 */

	function __construct($_id = null) {
		
		parent::__construct();
		
		if($_id)
			$this->id = $_id;
		else
			$this->id = substr(md5(uniqid('') . microtime()), 3, 5);
			
		$this->libraryPath = JURI::root().'components/com_analytics/'. $this->libraryPath;
		$this->swfObjectPath = JURI::root().'components/com_analytics/js/'. $this->swfObjectPath;

	}

	function addLabel($_text, $_x = 0, $_y = 0, $_config = array()) {

		$this->labels[] = array(
			"text" => $_text,
			"x" => $_x,
			"y" => $_y,
			"config" => $_config
		);

	}
	
	
	/**
	 * @param $xmlSettingsFile the $xmlSettingsFile to set
	 */
	function setXmlSettingsFile($xmlSettingsFile) {
		$this->xmlSettingsFile = $xmlSettingsFile;
	}

	/**
	 * Returns the HTML Code to insert on the page.
	 *
	 * @return	string
	 */
	function getCode($key = 1) {

		/*
		 * Create JS Code
		 */

		$code = '';

		// Background Color
		if(isset($this->config['background.color']) && is_array($this->config['background.color']))
			$bgColor = $this->config['background.color'][0];
		elseif(isset($this->config['background.color']))
			$bgColor = $this->config['background.color'];
		elseif(isset($this->config['background.alpha']))
			$bgColor = "transparent";
		else
			$bgColor = "#FFFFFF";
			unset($this->config['background.color']);
			unset($this->config['background.alpha']);
		// Width
		if(isset($this->config['width']))
			$width = $this->config['width'];
		else
			$width = 770;
		unset($this->config['width']);
		// Height
		if(isset($this->config['height']))
			$height = $this->config['height'];
		else
			$height = 400;
		unset($this->config['height']);
		//Preloader color
		if(isset($this->config['preloader_color']))
			$preloader_color = $this->config['preloader_color'];
		else
			$preloader_color = "#999999";
		unset($this->config['preloader_color']);
		$code = ''
			. 'var flashvars = {};'
			. 'flashvars.chart_id = "' . $this->id . '";'
			. 'flashvars.preloader_color = "' . $preloader_color. '";'
			. 'flashvars.path = "' . $this->getLibraryPath(). '";'
			. 'flashvars.settings_file = encodeURIComponent("'.$this->xmlSettingsFile.'?'.mktime().'");'
			. 'flashvars.chart_data = encodeURIComponent(\'' .  $this->getDataXml() . '\');'
			. 'flashvars.additional_chart_settings = encodeURIComponent(\'' . htmlspecialchars_decode($this->getConfigXml() ) . '\');'
			. 'var params = {};'
			. ($bgColor == "transparent" ? 'params.wmode="transparent";'  : 'params.bgcolor="'.$bgColor.'";')
			. 'swfobject.embedSWF("' . $this->getSwfPath() . '", "flashcontent'.$key.'", "' . $width . '", "' . $height . '", "8", "'.$this->getLibraryPath().'expressInstall.swf", flashvars, params, {});'
			;
		return $code;

	}

	/**
	 * Sets the config array. It should look like this:
	 * array(
	 *   "width" => 200,
	 *   "height" => 100,
	 *   "legend.enabled" => false
	 * )
	 *
	 * @param	array				$_config
	 * @param	bool				$_merge
	 */

	function setConfigAll($_config, $_merge = false) {

		if($_merge) {

			foreach($_config AS $key => $value)
				$this->config[$key] = $value;

		}
		else
			$this->config = $_config;

	}
	
	/**
	 * Sets one config variable.
	 *
	 * @param	string				$_key
	 * @param	mixed				$_value
	 */
	function setConfig($_key, $_value) {
		$this->config[$_key] = $_value;
	}
	
	/**
	 * Returns the config array.
	 *
	 * @return	array
	 */
	function getConfig() {

		return $this->config;

	}

	/**
	 * Returns the ready-to-use config as XML string or SimpleXml element.
	 *
	 * @param	bool				$_asString
	 * @return	mixed
	 */
	function getConfigXml($_asString = true) {
		
	    $val_array = '';
        $array = array();
	    foreach($this->config AS $key => $value) {
	        if($value === true || $value === false)
				$value = (int)$value;
	    
    	    $keyPath = (array)explode(".", $key);
        	$keyXML = array_shift($keyPath);
        	$i = true;
        	if( !empty($keyPath[0])) {
            	foreach ($keyPath as $k) {
            	    if ($i) {
            	        $val_array = array(array_pop($keyPath)=>$value);
            	        $i = false;
            	    } else
            	        $val_array = array(array_pop($keyPath) => $val_array);    
            	}
        	} else {
        	    $val_array = $value;
        	}
        	if(array_key_exists($keyXML,$array)){
        	    $array[$keyXML] = array_merge($array[$keyXML],$val_array);
        	}else
        	    $array[$keyXML]=$val_array;
	    }
	    
	    $settings = new JSimpleXML();
	    
		$settings->loadString( '<settings>'.$this->array2xml($array).'</settings>' );
		$settings = $settings->document;
		/*
		 * Add Labels
		 */

		if(count($this->labels) > 0) {

			$labels = &$settings->addChild("labels");

			foreach($this->labels AS $label) {

				$labelNode = &$labels->addChild("label");
				$tmp = &$labelNode->addChild("text");
				$tmp->setData( $label['text']);
				$tmp = &$labelNode->addChild("x");
				$tmp->setData( $label['x']);
				$tmp = &$labelNode->addChild("y");
				$tmp->setData($label['y']);

				foreach($label['config'] AS $key => $value) {
					$tmp = &$labelNode->addChild($key);
					$tmp->setData($value);
				}
			}
		}

		if($_asString) {
			return $settings->toString(false);
		}
		else
			return $settings;
	}

	/**
	 * Returns the path to the chart-specific swf file.
	 *
	 * @return	string
	 */
	function getLibraryPath() {
		return $this->libraryPath;
	}
	
	/**
	 * Creates a Xml from the settings array.
	 *
	 * @param	array				$_data
	 * @param	string				$_rootNode
	 * @return	SimpleXmlElement
	 */
	function array2xml($_data, $escape = true) {

	    $buff = '';
	    
		foreach ( $_data as $k => $v )
		{
			if ( !is_array( $v ) )
				$buff .= "<$k>" . ($escape ? (htmlspecialchars($v)) : $v ) . "</" . $k . ">";
			else
			{
				if ( isset( $v[0] ) )
				{
					foreach ( $v as $_k => $_v )
					{
						if ( is_array( $_v ) )
					 		$buff .= "<$k>" . $this->array2xml( $_v, $escape ) . "</" . $k . ">";
						else
							$buff .= "<$k>" . ($escape ? (htmlspecialchars($_v)) : $_v ) . "</" . $k . ">";
					}
				}
				else
					$buff .= "<$k>" . $this->array2xml( $v, $escape ) . "</" . $k .">";
			}
		}
		
		return $buff;
	}
	
}

?>