<?php
/**
* @copyright Copyright (C) 2006 - 2010 MightyExtensions (http://www.mightyextensions.com). All rights reserved.
* @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
**/

defined( '_JEXEC' ) or die( 'Restricted access' );

require_once(dirname(__FILE__) . "/amChart.php");

/*
 * Base class for AmLineChart and AmColumnChart.
 */
class AmLineBarChart extends AmChart {

	var $series = array();
	var $graphs = array();
	var $graphsConfigs = array();
	var $defaultConfig = array();
	
	/**
	 * Adds a new serie (value on the X axis).
	 *
	 * @param	string				$_id
	 * @param	string				$_title
	 * @param	array				$_config
	 */
	function addSerie($_id, $_title, $_config = array()) {

		$this->series[$_id] = array(
			"title" => $_title,
			"config" => $_config
		);

	}

	/**
	 * Adds a new graph (data line/bar).
	 *
	 * @param	string				$_id
	 * @param	string				$_title
	 * @param	array				$_data
	 * @param	array				$_config
	 */
	function addGraph($_id, $_title, $_data, $_config = array()) {

		$this->graphs[$_id] = array(
			"title" => $_title,
			"data" => $_data,
			"config" => $_config
		);

	}

	/**
	 * Sets the default config for graphs.
	 *
	 * @param	array				$_config
	 */
	function setDefaultConfig($_config) {
		
		$this->defaultConfig = $_config;
	}

	/**
	 * @see AmChart::getDataXml()
	 */
	function getDataXml($_asString = true) {

		$chart = new JSimpleXMLElement("chart");

		/*
		 * Series
		 */

		$series = &$chart->addChild("series");
		
		$this->addSeries($series);
		
		/*
		 * Graphs
		 */

		$graphs = &$chart->addChild("graphs");

	    foreach($this->graphs AS $key => $graph) {

			$graphNode = &$graphs->addChild("graph");

			// Set attributes

			$graphNode->addAttribute("gid", addslashes(str_replace('"',"'", $key)));
			
			$tmpGraph = clone($graphNode);
			
			$allAttributes = array_merge($this->defaultConfig, $graph['config']);

			foreach($allAttributes AS $key => $value) {
				$tmp = &$tmpGraph->addChild($key);
				$tmp->setData($value === true ? "true" : ($value === false ? "false" : $value));

			}
			
			$this->graphsConfigs[] = $tmpGraph;
			unset($tmpGraph);
			
			$graphNode->addAttribute("title", addslashes(str_replace('"',"'", $graph['title'])));

			// Set data

			foreach($graph['data'] AS $key => $value) {
			    
			    if ( !is_array($value) ) {
    				$valueNode = &$graphNode->addChild("value");
    				$valueNode->addAttribute("xid", addslashes(str_replace('"',"'", $key)));
    				$valueNode->setData(addslashes(str_replace('"',"'", $value)));
    				
			    } else {
			        
			        $valueNode = &$graphNode->addChild("value");
			        if ( !key_exists('xid',$value)){
			            $valueNode->addAttribute("xid", addslashes(str_replace('"',"'", $key)));
			        }
			        
			        foreach ($value as $_key => $_val) {
			            if( $_key != 'value' ) {
			        	    $valueNode->addAttribute($_key, addslashes(str_replace('"',"'", $_val)));
			            } else {
			                $valueNode->setData(addslashes(str_replace('"',"'", $value['value'])));
			            }
			        }
			        
			        if ( !key_exists('value',$value)){
			            $valueNode->setData(" 0 ");
			        }
			    }
			}
		}

		/*
		 * Return
		 */

		if($_asString) {
			return $chart->toString(false);
		}
		else
			return $chart;

	}
	
	/**
	 * Returns the ready-to-use config as XML string or SimpleXml element.
	 *
	 * @param	bool				$_asString
	 * @return	mixed
	 */
	function getConfigXml($_asString = true) {

		$val_array = '';
        $array = array();
	    foreach($this->config AS $key => $value) {
	        if($value === true || $value === false)
				$value = (int)$value;
	    
    	    $keyPath = (array)explode(".", $key);
        	$keyXML = array_shift($keyPath);
        	$i = true;
        	if( !empty($keyPath[0])) {
            	foreach ($keyPath as $k) {
            	    if ($i) {
            	        $val_array = array(array_pop($keyPath) => $value);
            	        $i = false;
            	    } else
            	        $val_array = array(array_pop($keyPath) => $val_array);    
            	}
        	} else {
        	    $val_array = $value;
        	}
        	if(array_key_exists($keyXML,$array)){
        	    $array[$keyXML] = array_merge($array[$keyXML],$val_array);
        	}else
        	    $array[$keyXML] = $val_array;
	    }
	    $settings = new JSimpleXML();
	    
		$settings->loadString( '<settings>'.$this->array2xml($array).'</settings>' );
		$settings = $settings->document;
		
		if(count($this->graphsConfigs) > 0) {

			$graphs = &$settings->addChild("graphs");
			$graphs->_children = $this->graphsConfigs;
		}

		/*
		 * Add Labels
		 */

		if(count($this->labels) > 0) {

			$labels = &$settings->addChild("labels");

			foreach($this->labels AS $label) {

				$labelNode = &$labels->addChild("label");
				$tmp = &$labelNode->addChild("text");
				$tmp->setData( $label['text']);
				$tmp = &$labelNode->addChild("x");
				$tmp->setData( $label['x']);
				$tmp = &$labelNode->addChild("y");
				$tmp->setData($label['y']);

				foreach($label['config'] AS $key => $value) {
					$tmp = &$labelNode->addChild($key);
					$tmp->setData($value);
				}
			}
		}

		if($_asString) {
			return $settings->toString(false);
		}
		else
			return $settings;
	}

}

?>